# Example input:
# {'mail.example.com.': {'A': ['192.0.2.1', '192.0.2.2']}, 'www.example.com.': {'A': ['192.0.2.1', '192.0.2.2']}}
{% for name in template_data -%}
{% for rdtype in template_data[name] -%}
# RecordSet: {{name}} / {{rdtype}}
{% for record in template_data[name][rdtype] -%}
{{record}} {{name}}
{% endfor %}
{% endfor %}
{%- endfor %}
